Automatic Speech Recogniton REST API
====================================

![logo](media/asr_logo.jpg)

A simple REST API around a python package for automatic speech recognition (ASR).
 
This code has been developed by [ZKM | Hertz-Lab](https://zkm.de/en/about-the-zkm/organization/hertz-lab) as part of the project [»The Intelligent Museum«](#the-intelligent-museum).

Copyright (c) 2022-2025 ZKM | Karlsruhe.  
Copyright (c) 2022 Paul Bethge.  
Copyright (c) 2025 Dan Wilcox.

For information on usage and redistribution, and for a DISCLAIMER OF ALL
WARRANTIES, see the file, "LICENSE.txt," in this distribution.

BSD Simplified License.

Description
-----------

This repository simplifies running an ASR REST API on a local machine. It is intended to be used with Docker. The API expects a JSON containing a base64 encoded audio file.

### Structure

* /backend: contains the code for the API, there are different backends utilizing different python packages
* /client: contains simple clients for the chosen backend

Requirements
------------

Software:
* Docker

Operating systems/platforms:
* macOS
* Linux
* Windows

Installation & Build
--------------------

There are two backends that can be chosen from:
* huggingsound: contains wav2vec2 models trained on different languages
* whisper: a powerful multilingual model that detects the language and transcribes the text (recommended)

Clone this repository and enter the backend directory, in this case "whisper":

    cd automatic-speech-recognition/backend/whisper

Build the docker image:

    make

If successful, run with default settings (port 6969, GPU enabled):

    make run

To set a different port or disable GPU usage, use the makefile variable overrides:

    make run PORT=7000 GPU=false

To test locally on port 8000:

    make test

Additional makefile targets wrap docker commands:
* start
* stop
* remove
* logs

The makefile clean target will stop, remove, and remove the built image:

    make clean

Usage
-----

### Test Web Interface 

Open `client/XXXX/client.html` in your browser. Select a wav or mp3 file and run the computation by pressing the start button.

Afterwards you will find the transcribed text and some meta data in a field below.

_Note: Both clients use localhost:6969 by default. You may need to change the `host` or `port` variables in the html files if using a different server._

### JSON API

Port: 6969

Endpoint: transcribe\_audio

API type: POST with JSON data, base-encoded mp3 audio file, returns JSON data

JSON data format: object with following keys
* sample: audio MP3 encoded as a base64 string

Example:

```json
{"sample": "/9j/4AAQSkZJRgAB..."}
```

Test API with curl:

```shell
curl -H "Content-Type: application/json" -X POST \
    -d '{"sample": "/9j/4AAQSkZJRgAB..."}' \
    http://localhost:6969/transcribe_audio
```

To encode a file for testing on the commandline, try `base64` if available:

    base64 test.mp3

Successful output is a JSON object with the following keys
* text: detected text in original language
* language: language code, ie. "en", "de", etc
* probs: confidence as a probability 0-100%

References / Further Reading
----------------------------

* [huggingsound](https://github.com/jonatasgrosman/huggingsound)
* [whisper](https://github.com/openai/whisper)

Contribute
----------

Contributions are very welcome! Clone or fork the repo, then request a pull / merge.

If you find any bugs or suggestions please log them here as well.

The Intelligent Museum
----------------------

An artistic-curatorial field of experimentation for deep learning and visitor participation

The [ZKM | Center for Art and Media](https://zkm.de/en) and the [Deutsches Museum Nuremberg](https://www.deutsches-museum.de/en/nuernberg/information/) cooperate with the goal of implementing an AI-supported exhibition. Together with researchers and international artists, new AI-based works of art will be realized during the next four years (2020-2023).  They will be embedded in the AI-supported exhibition in both houses. The Project „The Intelligent Museum” is funded by the Digital Culture Programme of the [Kulturstiftung des Bundes](https://www.kulturstiftung-des-bundes.de/en) (German Federal Cultural Foundation) and funded by the [Beauftragte der Bundesregierung für Kultur und Medien](https://www.bundesregierung.de/breg-de/bundesregierung/staatsministerin-fuer-kultur-und-medien) (Federal Government Commissioner for Culture and the Media).

As part of the project, digital curating will be critically examined using various approaches of digital art. Experimenting with new digital aesthetics and forms of expression enables new museum experiences and thus new ways of museum communication and visitor participation. The museum is transformed to a place of experience and critical exchange.

![Logo](media/Logo_ZKM_DMN_KSB.png)
